@@ -23,6 +23,8 @@ GENERIC_SWITCH_USER_MAX_SESSIONS=${GENERIC_SWITCH_USER_MAX_SESSIONS:-0}
 # 0 would mean wait forever
 GENERIC_SWITCH_DLM_ACQUIRE_TIMEOUT=${GENERIC_SWITCH_DLM_ACQUIRE_TIMEOUT:-120}
 
+enable_python3_package networking-generic-switch
+
 if ( [[ "$GENERIC_SWITCH_USER_MAX_SESSIONS" -gt 0 ]] ) && (! is_service_enabled etcd3); then
     die $LINENO "etcd3 service must be enabled to use coordination features of networking-generic-switch"
 fi
@@ -217,6 +219,16 @@ if is_service_enabled generic_switch; then
     elif [[ "$1" == "stack" && "$2" == "post-config" ]]; then
         # Configure after the other layer 1 and 2 services have been configured
         echo_summary "Configuring Generic_switch ML2"
+
+        # Source ml2 plugin, set default config
+        if is_service_enabled neutron; then
+            source $RC_DIR/lib/neutron_plugins/ml2
+            Q_PLUGIN_CONF_PATH=etc/neutron/plugins/ml2
+            Q_PLUGIN_CONF_FILENAME=ml2_conf.ini
+            Q_PLUGIN_CONF_FILE="/${Q_PLUGIN_CONF_PATH}/${Q_PLUGIN_CONF_FILENAME}"
+            Q_PLUGIN_CLASS="ml2"
+        fi
+
         configure_generic_switch
     elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
         if is_service_enabled tempest; then
