@@ -20,6 +20,7 @@ import netmiko
 from oslo_config import cfg
 from oslo_log import log as logging
 import paramiko
+import six
 import tenacity
 from tooz import coordination
 
@@ -32,6 +33,32 @@ LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 
 
+def check_output(operation):
+    """Returns a decorator that checks the output of an operation.
+
+    :param operation: Operation being attempted. One of 'add network',
+        'delete network', 'plug port', 'unplug port'.
+    """
+    def decorator(func):
+        """The real decorator."""
+
+        @six.wraps(func)
+        def wrapper(self, *args, **kwargs):
+            """Wrapper that checks the output of an operation.
+
+            :returns: The return value of the wrapped method.
+            :raises: GenericSwitchNetmikoConfigError if the driver detects that
+                an error has occurred.
+            """
+            output = func(self, *args, **kwargs)
+            self.check_output(output, operation)
+            return output
+
+        return wrapper
+
+    return decorator
+
+
 class NetmikoSwitch(devices.GenericSwitchDevice):
 
     ADD_NETWORK = None
@@ -46,8 +73,19 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
 
     REMOVE_NETWORK_FROM_TRUNK = None
 
+    ENABLE_PORT = None
+
+    DISABLE_PORT = None
+
     SAVE_CONFIGURATION = None
 
+    ERROR_MSG_PATTERNS = ()
+    """Sequence of error message patterns.
+
+    Sequence of re.RegexObject objects representing patterns to check for in
+    device output that indicate a failure to apply configuration.
+    """
+
     def __init__(self, device_cfg):
         super(NetmikoSwitch, self).__init__(device_cfg)
         device_type = self.config.get('device_type', '')
@@ -151,6 +189,7 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
         LOG.debug(output)
         return output
 
+    @check_output('add network')
     def add_network(self, segmentation_id, network_id):
         # NOTE(zhenguo): Remove dashes from uuid as on most devices 32 chars
         # is the max length of vlan name.
@@ -162,8 +201,9 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
             cmds += self._format_commands(self.ADD_NETWORK_TO_TRUNK,
                                           port=port,
                                           segmentation_id=segmentation_id)
-        self.send_commands_to_device(cmds)
+        return self.send_commands_to_device(cmds)
 
+    @check_output('delete network')
     def del_network(self, segmentation_id, network_id):
         # NOTE(zhenguo): Remove dashes from uuid as on most devices 32 chars
         # is the max length of vlan name.
@@ -176,10 +216,13 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
         cmds += self._format_commands(self.DELETE_NETWORK,
                                       segmentation_id=segmentation_id,
                                       network_id=network_id)
-        self.send_commands_to_device(cmds)
+        return self.send_commands_to_device(cmds)
 
+    @check_output('plug port')
     def plug_port_to_network(self, port, segmentation_id):
         cmds = []
+        if self._disable_inactive_ports() and self.ENABLE_PORT:
+            cmds += self._format_commands(self.ENABLE_PORT, port=port)
         ngs_port_default_vlan = self._get_port_default_vlan()
         if ngs_port_default_vlan:
             cmds += self._format_commands(
@@ -190,8 +233,9 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
             self.PLUG_PORT_TO_NETWORK,
             port=port,
             segmentation_id=segmentation_id)
-        self.send_commands_to_device(cmds)
+        return self.send_commands_to_device(cmds)
 
+    @check_output('unplug port')
     def delete_port(self, port, segmentation_id):
         cmds = self._format_commands(self.DELETE_PORT,
                                      port=port,
@@ -206,7 +250,9 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
                 self.PLUG_PORT_TO_NETWORK,
                 port=port,
                 segmentation_id=ngs_port_default_vlan)
-        self.send_commands_to_device(cmds)
+        if self._disable_inactive_ports() and self.DISABLE_PORT:
+            cmds += self._format_commands(self.DISABLE_PORT, port=port)
+        return self.send_commands_to_device(cmds)
 
     def send_config_set(self, net_connect, cmd_set):
         """Send a set of configuration lines to the device.
@@ -233,3 +279,28 @@ class NetmikoSwitch(devices.GenericSwitchDevice):
                 LOG.warning("Saving config is not supported for %s,"
                             " all changes will be lost after switch"
                             " reboot", self.config['device_type'])
+
+    def check_output(self, output, operation):
+        """Check the output from the device following an operation.
+
+        Drivers should implement this method to handle output from devices and
+        perform any checks necessary to validate that the configuration was
+        applied successfully.
+
+        :param output: Output from the device.
+        :param operation: Operation being attempted. One of 'add network',
+            'delete network', 'plug port', 'unplug port'.
+        :raises: GenericSwitchNetmikoConfigError if the driver detects that an
+            error has occurred.
+        """
+        if not output:
+            return
+
+        for pattern in self.ERROR_MSG_PATTERNS:
+            if pattern.search(output):
+                msg = ("Found invalid configuration in device response. "
+                       "Operation: %(operation)s. Output: %(output)s" %
+                       {'operation': operation, 'output': output})
+                raise exc.GenericSwitchNetmikoConfigError(
+                    config=device_utils.sanitise_config(self.config),
+                    error=msg)
